import logging
import threading
import time
from concurrent.futures import Future
from datetime import datetime, timezone
from threading import Thread
from time import sleep
from typing import Iterable, Optional

from confluent_kafka import Producer, Consumer, Message

from eventy.config import SERVICE_NAME
from eventy.messaging.errors import MessagingError
from eventy.messaging.store import RecordStore, Cursor
from eventy.record import Record
from eventy.serialization import RecordSerializer

__all__ = [
    'CKStore',
]

from eventy.trace_id import local_trace

logger = logging.getLogger(__name__)


class CKStore(RecordStore):
    """
    Kafka implementation of a record store, using Confluent kafka library
    """

    transactional_producer: Producer
    immediate_producer: Producer
    consumer: Consumer
    poll_thread: Thread

    def __init__(
        self,
        serializer: RecordSerializer,
        bootstrap_servers: list[str],
        group_id: Optional[str] = None,
        transactional_id: Optional[str] = None,
        sasl_username: Optional[str] = None,
        sasl_password: Optional[str] = None,
    ) -> None:
        """
        Initialize the store

        :param serializer: Record serializer (read and write)
        :param bootstrap_servers: Kafka bootstrap servers (producer and consumer)
        :param group_id: Kafka group id (consumer)
        :param transactional_id: Kafka transactional id (producer), default to unique id
        :param sasl_username: Optional username (if SASL PLAIN kafka connection, producer and consumer)
        :param sasl_password: Optional password (if SASL PLAIN kafka connection, producer and consumer)
        """
        # Initialize read_from services
        super().__init__()
        # Kafka config
        self.serializer = serializer
        self.bootstrap_servers = bootstrap_servers
        self.group_id = group_id or SERVICE_NAME
        self.transactional_id = transactional_id or f'{SERVICE_NAME}-{str(100 * int(datetime.now(timezone.utc).timestamp()))}'
        self.sasl_username = sasl_username
        self.sasl_password = sasl_password
        # Other attributes
        self.initialized = False
        self.in_transaction = False
        self.cancelled = False

    def register_topic(self, topic: str, cursor: Cursor = Cursor.ACKNOWLEDGED):
        if cursor != Cursor.ACKNOWLEDGED:
            raise NotImplementedError(f"Only {Cursor.ACKNOWLEDGED} implemented for now.")
        super().register_topic(topic, cursor)
        logger.info(f"Registered topic {topic}: {cursor}.")

    def _check_initialized(self) -> None:
        if self.initialized:
            return

        logger.info(f"Will initialize Kafka producer and consumer.")

        # configs
        transactional_producer_config = {
            'bootstrap.servers': ','.join(self.bootstrap_servers),
            'transactional.id': self.transactional_id,
            'enable.idempotence': 'true',
        }
        immediate_producer_config = {
            'bootstrap.servers': ','.join(self.bootstrap_servers),
            'enable.idempotence': 'true',
        }
        consumer_config = {
            'bootstrap.servers': ','.join(self.bootstrap_servers),
            'group.id': self.group_id,
            'enable.auto.commit': 'false',
            'auto.offset.reset': 'earliest',
            'isolation.level': 'read_committed',
        }

        # SASL authentication
        if self.sasl_username and self.sasl_password:
            for config in [
                transactional_producer_config,
                immediate_producer_config,
                consumer_config,
            ]:
                config.update(
                    {
                        'sasl_mechanism': 'PLAIN',
                        'sasl_plain_username': self.sasl_username,
                        'sasl_plain_password': self.sasl_password,
                    }
                )

        # producers
        logger.info(f"Initializing transactional Kafka producer with config {transactional_producer_config}.")
        self.transactional_producer = Producer(transactional_producer_config)
        self.transactional_producer.init_transactions()
        logger.debug(f"Initialized transactional Kafka producer.")

        logger.info(f"Initializing immediate Kafka producer with config {immediate_producer_config}.")
        self.immediate_producer = Producer(immediate_producer_config)
        logger.debug(f"Initialized immediate Kafka producer.")

        # consumer
        consumer_topics = [topic for topic in self.topics]
        logger.info(f"Initializing Kafka consumer with config {consumer_config} and topics {consumer_topics}.")
        self.consumer = Consumer(consumer_config)
        if consumer_topics:
            self.consumer.subscribe(consumer_topics)
        logger.debug(f"Initialized Kafka consumer.")

        # poll loop
        current_thread = threading.current_thread()

        def poll_loop():

            while True:
                if self.cancelled:
                    finish_reason = "cancelled"
                    break
                elif not current_thread.is_alive():
                    finish_reason = "parent thread not alive"
                    break
                else:
                    try:
                        immediate_polled = self.immediate_producer.poll(0.005)
                        if immediate_polled:
                            logger.debug(f"Immediate producer polled {immediate_polled} messages.")
                        transactional_polled = self.transactional_producer.poll(0.005)
                        if transactional_polled:
                            logger.debug(f"Transactional producer polled {transactional_polled} messages.")
                    except Exception as e:
                        logger.warning(f"Kafka exception in poll loop: {e}")
                        sleep(30)
            logger.warning(f"Kafka poll loop stopped. Reason: {finish_reason}.")

        self.poll_thread = Thread(
            target=poll_loop,
            name="KafkaPollLoop",
        )

        logger.debug(f"Will start Kafka poll loop thread.")
        self.poll_thread.start()

        self.initialized = True
        logger.debug(f"Record Store initialized.")

    def read(
        self,
        max_count: int = 1, timeout_ms: Optional[int] = None, auto_ack: bool = False
    ) -> Iterable[Record]:
        """
        Reads records from kafka.

        :raise MessagingError: if there is an error reading from kafka.

        :param max_count:
        :param timeout_ms:
        :param auto_ack:
        :return:
        """
        self._check_initialized()

        try:
            logger.debug(f"Reading at most {max_count} records from Kafka.")
            messages: list[Message] = self.consumer.consume(max_count, _ms2sec(timeout_ms))
            logger.debug(f"Read {len(messages)} messages from Kafka.")
        except Exception as e:
            raise MessagingError(f"Error consuming messages from kafka.") from e

        for message in messages:
            if message.error():
                logger.warning(f"Message has error: {message.error()}.")
                continue
            logger.debug(f"New message read {message.topic()}:{message.offset()}")
            try:
                # noinspection PyArgumentList
                # PyCharm is wrong, there is no payload arg
                encoded = message.value()
                record = self.serializer.decode(encoded)
            except Exception as e:
                raise MessagingError(f"Error decoding message {message.topic()}:{message.offset()}.") from e
            logger.info(f"New record {record.qualified_name} from kafka message {message.topic()}:{message.offset()}.")
            yield record
            if auto_ack:
                try:
                    self.consumer.commit(message=message, asynchronous=False)
                except Exception as e:
                    raise MessagingError(f"Error committing message {message.topic()}:{message.offset()}.") from e

    def ack(self, timeout_ms=None) -> None:
        self._check_initialized()

        if self.in_transaction:
            logger.info(f"Ack read messages in transaction.")
            try:
                self.transactional_producer.send_offsets_to_transaction(
                    self.consumer.position(self.consumer.assignment()),
                    self.consumer.consumer_group_metadata(),
                    _ms2sec(timeout_ms)
                )
            except Exception as e:
                raise MessagingError(f"Error sending offsets to transaction.") from e
        else:
            logger.info(f"Ack read messages outside transaction.")
            try:
                self.consumer.commit(asynchronous=False)
            except Exception as e:
                raise MessagingError(f"Error committing message.") from e

    def _write(self, producer: Producer, record: Record, topic: str, timeout_ms=None) -> None:

        # handle callback
        future_result: Future = Future()

        def ack(err, msg):
            with local_trace(correlation_id=record.correlation_id):
                if err:
                    logger.debug(f"Produced with errors: {err}.")
                    future_result.set_exception(MessagingError(err))
                else:
                    logger.debug(f"Produced successfully.")
                    future_result.set_result(msg)

        # produce message
        logger.debug(f"Will produce record {record.qualified_name} on {topic}.")
        producer.produce(
            topic=topic,
            value=self.serializer.encode(record),
            key=record.partition_key,
            on_delivery=ack
        )

        # wait callback
        total_time_ms = 0.0
        while not future_result.done():
            time_ms = 1 + total_time_ms / 10
            time.sleep(time_ms / 1000)  # in secs
            total_time_ms = total_time_ms + time_ms
            if timeout_ms is not None and total_time_ms > timeout_ms:
                raise TimeoutError(f"Write timed out {int(total_time_ms)} ms.")

        # handle exceptions
        exception = future_result.exception()
        if exception:
            raise MessagingError(exception)

        # handle interrupted thread
        result = future_result.result()
        if not result:
            raise MessagingError(f"Could not write. Was the thread was interrupted?")

    def write(self, record: Record, topic: str, timeout_ms=None) -> None:
        self._check_initialized()

        if self.in_transaction:
            self._write(self.transactional_producer, record, topic, timeout_ms)
        else:
            self._write(self.immediate_producer, record, topic, timeout_ms)

    def write_now(self, record: Record, topic: str, timeout_ms=None) -> None:
        self._check_initialized()

        self._write(self.immediate_producer, record, topic, timeout_ms)

    def start_transaction(self) -> None:
        self._check_initialized()

        if self.in_transaction:
            raise MessagingError(f"Already in a transaction.")
        self.transactional_producer.begin_transaction()
        self.in_transaction = True

    def commit(self, timeout_ms: Optional[int] = None) -> None:
        self._check_initialized()

        if not self.in_transaction:
            raise MessagingError(f"Not in a transaction.")
        self.transactional_producer.commit_transaction(_ms2sec(timeout_ms))
        self.in_transaction = False

    def abort(self, timeout_ms: Optional[int] = None) -> None:
        self._check_initialized()

        if not self.in_transaction:
            raise MessagingError(f"Not in a transaction.")
        self.transactional_producer.abort_transaction(_ms2sec(timeout_ms))
        self.in_transaction = False

    def __del__(self):
        if self.initialized:
            logger.info(f"Waiting poll thread to join.")
            self.cancelled = True
            self.poll_thread.join()
            logger.info(f"Poll thread joined.")


def _ms2sec(timeout_ms: Optional[int]) -> float:
    timeout_sec = -1.0
    if timeout_ms:
        timeout_sec = timeout_ms / 1000
    return timeout_sec



from enum import Enum
from functools import wraps
from typing import Callable, Iterable, Optional

from eventy.record import Record, RecordType

__all__ = [
    'Guarantee',
    'Handler',
    'Agent',
    'handler',
    'handle_event',
    'handle_response',
    'handle_request',
]


class Guarantee(Enum):
    """
    Delivery guarantee for handlers
    """
    AT_LEAST_ONCE = 'AT_LEAST_ONCE'
    # alias
    ALO = 'AT_LEAST_ONCE'

    AT_MOST_ONCE = 'AT_MOST_ONCE'
    # alias
    AMO = 'AT_MOST_ONCE'

    EXACTLY_ONCE = 'EXACTLY_ONCE'
    # alias
    EOS = 'EXACTLY_ONCE'


class Handler:
    """
    Record handler
    """

    def __init__(
        self,
        service_name: Optional[str],
        record_type: RecordType,
        record_name: str,
        delivery_guarantee: Guarantee,
    ):
        self._service_name = service_name
        self._delivery_guarantee = delivery_guarantee
        self._record_type = record_type
        self._record_name = record_name

    @property
    def service_name(self) -> Optional[str]:
        return self._service_name

    @property
    def record_type(self) -> RecordType:
        return self._record_type

    @property
    def record_name(self) -> str:
        return self._record_name

    @property
    def delivery_guarantee(self) -> Guarantee:
        return self._delivery_guarantee

    def handle_record(self, record: Record) -> Iterable[Record]:
        raise NotImplementedError

    def __str__(self) -> str:
        record_info = f"{self._record_name} ({self._record_type})"
        if self._service_name is None:
            service_info = "current app service"
        else:
            service_info = self._service_name
        return f'<{self.__class__.__name__} for {record_info} from {service_info}>'


class Agent:
    """
    An agent can define methods to handle records
    """

    @property
    def handlers(self) -> Iterable[Handler]:
        for attribute_name in self.__dir__():
            attribute = getattr(self, attribute_name)
            if isinstance(attribute, _AgentHandler):
                agent_handler = attribute
                agent_handler.set_agent(self)
                yield agent_handler


class _FunctionHandler(Handler):
    """
    Record handler
    """

    def __init__(
        self,
        service_name: Optional[str],
        record_type: RecordType,
        record_name: str,
        delivery_guarantee: Guarantee,
        handle_function: Callable[[Record], Iterable[Record]],
    ) -> None:
        super().__init__(
            service_name=service_name,
            record_type=record_type,
            record_name=record_name,
            delivery_guarantee=delivery_guarantee,
        )
        self._handle_function = handle_function

    def handle_record(self, record: Record) -> Iterable[Record]:
        yield from self._handle_function(record)

    def __call__(self, record: Record) -> Iterable[Record]:
        """
        Override __call__ so the method can be called directly as if it was not decorated.
        """
        yield from self._handle_function(record)


class _AgentHandler(Handler):
    """
    Record handler for an Agent method
    """

    def __init__(
        self,
        service_name: Optional[str],
        record_type: RecordType,
        record_name: str,
        delivery_guarantee: Guarantee,
        handle_method: Callable[[Agent, Record], Iterable[Record]],
    ) -> None:
        super().__init__(
            service_name=service_name,
            record_type=record_type,
            record_name=record_name,
            delivery_guarantee=delivery_guarantee,
        )
        self._handle_method = handle_method
        self._handle_function: Optional[Callable[[Record], Iterable[Record]]] = None

    def set_agent(self, agent: Agent) -> None:
        method = self._handle_method

        @wraps(method)
        def function(record: Record) -> Iterable[Record]:
            yield from self._handle_method(agent, record)

        self._handle_function = function

    def handle_record(self, record: Record) -> Iterable[Record]:
        if self._handle_function is None:
            raise AttributeError(f"Agent need to be set before usage.")
        else:
            yield from self._handle_function(record)

    def __call__(self, agent_self: Agent, record: Record) -> Iterable[Record]:
        """
        Override __call__ so the method can also be called directly as if it was not decorated.
        """
        yield from self._handle_method(agent_self, record)


def handler(type: RecordType, service: Optional[str], name: str, guarantee: Guarantee):
    """
    Decorator to create handlers from functions or agent methods
    """

    def decorator(function_or_method):
        # Agent method
        if '.' in function_or_method.__qualname__:
            method = function_or_method
            return _AgentHandler(
                service_name=service,
                record_type=type,
                record_name=name,
                delivery_guarantee=guarantee,
                handle_method=method,
            )
        # Simple function
        else:
            function = function_or_method
            return _FunctionHandler(
                service_name=service,
                record_type=type,
                record_name=name,
                delivery_guarantee=guarantee,
                handle_function=function,
            )

    return decorator


def handle_event(service: str, name: str, guarantee: Guarantee = Guarantee.EXACTLY_ONCE):
    return handler(RecordType.EVENT, service, name, guarantee)


def handle_request(name: str, guarantee: Guarantee = Guarantee.EXACTLY_ONCE):
    return handler(RecordType.REQUEST, None, name, guarantee)


def handle_response(service: str, name: str, guarantee: Guarantee = Guarantee.EXACTLY_ONCE):
    return handler(RecordType.RESPONSE, service, name, guarantee)



import logging
from enum import Enum
from typing import Iterable, Optional, Dict

from eventy.messaging.errors import MessagingError
from eventy.record import Record

__all__ = [
    'RecordStore',
    'Cursor',
    'RecordWriteStore',
]

logger = logging.getLogger(__name__)


class Cursor(Enum):
    """
    Read cursor
    """

    BEGIN = 0
    ACKNOWLEDGED = 1
    CURRENT = 2


class RecordWriteStore:
    def write(self, record: Record) -> None:
        raise NotImplementedError


class RecordStore:
    """
    A RecordStore is a transactional read and write store
    """

    def __init__(self):
        self.topics: Dict[str, Cursor] = dict()

    def register_topic(self, topic: str, cursor: Cursor = Cursor.ACKNOWLEDGED):
        if topic in self.topics:
            raise MessagingError(f"Topic {topic} was already added.")
        self.topics[topic] = cursor

    def read(
        self,
        max_count: int = 1,
        timeout_ms: Optional[int] = None,
        auto_ack: bool = False
    ) -> Iterable[Record]:
        """
        Fetch between 0 and max_count records
        """
        raise NotImplementedError

    def ack(self, timeout_ms=None) -> None:
        """
        Acknowledge all fetched records
        """
        raise NotImplementedError

    def write(self, record: Record, topic: str, timeout_ms=None) -> None:
        """
        Add to current transaction if a transaction was started,
        and write immediately otherwise
        """
        raise NotImplementedError

    def write_now(self, record: Record, topic: str, timeout_ms=None) -> None:
        """
        Write immediately otherwise, even if a transaction was started
        """
        raise NotImplementedError

    def start_transaction(self) -> None:
        """
        Start a new transaction
        """
        raise NotImplementedError

    def commit(self, timeout_ms: Optional[int] = None) -> None:
        """
        Ack all consumed records, and produce all records added to transaction
        """
        raise NotImplementedError

    def abort(self, timeout_ms: Optional[int] = None) -> None:
        """
        Roll back to previously acknowledged record, discard records in current transaction
        """
        raise NotImplementedError



import logging
import threading
import time
from threading import Thread
from typing import Optional, Iterable

from eventy.messaging.agent import Agent, Handler, Guarantee
from eventy.messaging.service import Service
from eventy.messaging.store import Cursor, RecordStore, RecordWriteStore
from eventy.record import Record, Response, RecordType
from eventy.trace_id.local import local_trace

__all__ = [
    'EventyApp',
]

logger = logging.getLogger(__name__)


class EventyApp:
    """
    EventyApp is the messaging application.
    """

    def __init__(
        self,
        record_store: RecordStore,
        app_service: Service,
        ext_services: Iterable[Service] = None,
        agents: Iterable[Agent] = None,
        handlers: Iterable[Handler] = None,
        read_batch_size: int = 1,
        read_timeout_ms: Optional[int] = None,
        read_wait_time_ms: Optional[int] = None,
        write_timeout_ms: Optional[int] = None,
        ack_timeout_ms: Optional[int] = None,
        commit_timeout_ms: Optional[int] = None,
    ):
        self._record_store = record_store
        self._app_service = app_service
        self._ext_services = ext_services or []
        self._all_handlers = list()
        if handlers is not None:
            for handler in handlers:
                self._all_handlers.append(handler)
        if agents is not None:
            for agent in agents:
                for handler in agent.handlers:
                    self._all_handlers.append(handler)

        self._read_batch_size = read_batch_size
        self._read_timeout_ms = read_timeout_ms
        self._write_timeout_ms = write_timeout_ms
        self._ack_timeout_ms = ack_timeout_ms
        self._commit_timeout_ms = commit_timeout_ms
        self._read_wait_time_ms = read_wait_time_ms or 0

        self._write_store = _AppRecordWriteStore(self)
        self._started = False
        self._cancelled = False

    def _get_service_for_name(self, service_name: Optional[str]):
        if service_name is None:
            return self._app_service
        for service in self._ext_services:
            if service.name == service_name:
                return service
        raise ValueError(f"Service {service_name} not found.")

    def _get_topics_to_register(self) -> list[str]:
        topics: set[str] = set()
        topics.add(self._app_service.request_topic)
        for handler in self._all_handlers:
            service = self._get_service_for_name(handler.service_name)
            topic = service.topic_for(handler.record_type)
            logger.info(f"Found handler {handler}. Will listen topic {topic}.")
            topics.add(topic)
        return list(sorted(topics))

    def _get_handlers_for_record(self, record: Record, guarantee: Guarantee) -> list[Handler]:
        handlers: list[Handler] = list()
        for handler in self._all_handlers:
            if (
                handler.record_type == record.type
                and handler.record_name == record.name
                and handler.delivery_guarantee == guarantee
                and self._get_service_for_name(handler.service_name).namespace == record.namespace
            ):
                if isinstance(record, Response):
                    if record.destination == self._app_service.namespace:
                        handlers.append(handler)
                else:
                    handlers.append(handler)
        return handlers

    def _get_topic_for_record(self, record: Record) -> str:
        if record.type == RecordType.EVENT or record.type == RecordType.RESPONSE:
            return self._app_service.topic_for(record.type)
        for service in self._ext_services:
            if service.namespace == record.namespace:
                return service.topic_for(record.type)
        raise ValueError(f"No topic for record {record}.")

    @property
    def write_store(self) -> RecordWriteStore:
        return self._write_store

    def _process_one_record_one_handler(self, record: Record, handler: Handler) -> None:
        logger.debug(
            f"Processing record {record.qualified_name} data={record.data} with handler {handler}."
        )
        for output_record in handler.handle_record(record):
            topic = self._get_topic_for_record(output_record)
            logger.debug(
                f"Writing output record {record.qualified_name} on topic {topic} with data={record.data}."
            )
            self._record_store.write(output_record, topic, self._write_timeout_ms, )

    def _process_one_batch(self) -> None:
        records = list(
            self._record_store.read(
                max_count=self._read_batch_size,
                timeout_ms=self._read_timeout_ms,
                auto_ack=False,
            )
        )
        if records:
            logger.debug(f"Received {len(records)} new records.")
        else:
            time.sleep(self._read_wait_time_ms / 1000)
            return

        # At Least Once
        for record in records:
            with local_trace(correlation_id=record.correlation_id):
                for handler in self._get_handlers_for_record(record, Guarantee.AT_LEAST_ONCE):
                    self._process_one_record_one_handler(record, handler)

        # Exactly Once
        self._record_store.start_transaction()
        self._record_store.ack(self._ack_timeout_ms)
        for record in records:
            with local_trace(correlation_id=record.correlation_id):
                for handler in self._get_handlers_for_record(record, Guarantee.EXACTLY_ONCE):
                    self._process_one_record_one_handler(record, handler)
        self._record_store.commit(self._commit_timeout_ms)

        # At Most Once
        for record in records:
            with local_trace(correlation_id=record.correlation_id):
                for handler in self._get_handlers_for_record(record, Guarantee.AT_MOST_ONCE):
                    self._process_one_record_one_handler(record, handler)

    def run(self, keep_alive=False) -> None:
        """
        Run in a separate thread.

        :param keep_alive: If True, the app will not exit until the calling thread is finished.
        """
        logger.info(f"Starting Eventy App <{self._app_service.name}>.")

        topics_to_register = self._get_topics_to_register()
        logger.info(
            f"Starting Eventy App <{self._app_service.name}>: registering {len(topics_to_register)} topics: {', '.join(topics_to_register)}."
        )
        for topic in topics_to_register:
            logger.debug(f"Will register to topic: {topic}.")
            try:
                self._record_store.register_topic(topic, Cursor.ACKNOWLEDGED)
            except Exception as e:
                logger.error(f"Failed to register to topic {topic}: {e}")

        current_thread = threading.current_thread()

        def run_loop():
            while True:
                if self._cancelled:
                    logger.info(f"Eventy App <{self._app_service.name}> has been cancelled. Will terminate App.")
                    finish_reason = "cancelled"
                    break
                elif not keep_alive and not current_thread.is_alive():
                    logger.info(
                        f"Eventy App <{self._app_service.name}> calling thread is not alive. Will terminate App."
                    )
                    finish_reason = "parent thread not alive"
                    break
                else:
                    try:
                        self._process_one_batch()
                    except Exception as e:
                        logger.exception("Failed to process batch. Will terminate App.", e)
                        finish_reason = "failed to process batch"
                        break
            logger.warning(f"Stopped processing records <{self._app_service.name}>. Reason: {finish_reason}.")

        self.process_thread = Thread(
            name=f"ProcessLoop-{self._app_service.name}",
            target=run_loop,
        )

        logger.info(f"Starting Eventy App <{self._app_service.name}>: starting processing thread.")
        self.process_thread.start()
        self._started = True

        logger.info(f"Starting Eventy App <{self._app_service.name}>: started.")

    def cancel(self) -> None:
        if self._started and not self._cancelled:
            logger.info(f"Waiting process thread to join.")
            self._cancelled = True
            self.process_thread.join()
            logger.info(f"Process thread joined.")

    def __del__(self):
        logger.info(f"Deleting Eventy App <{self._app_service.name}>.")
        self.cancel()


class _AppRecordWriteStore(RecordWriteStore):
    def __init__(self, app: EventyApp):
        self._app = app

    def write(self, record: Record) -> None:
        with local_trace(correlation_id=record.correlation_id):
            topic = self._app._get_topic_for_record(record)
            logger.debug(f"Writing record {record.qualified_name} data={record.data} to topic: {topic}.")
            self._app._record_store.write_now(
                record,
                topic,
                self._app._write_timeout_ms,
            )



# Copyright (c) Qotto, 2021

"""
Eventy messaging API

The messaging API itself is backend-agnostic, there is currently only a Kafka backend implemented.
"""
from eventy.messaging.agent import Agent, handle_event, handle_request, handle_response, Guarantee
from eventy.messaging.app import EventyApp
from eventy.messaging.errors import MessagingError
from eventy.messaging.service import Service
from eventy.messaging.store import RecordStore, RecordWriteStore, Cursor

__all__ = [
    # base
    'Service',
    # store
    'RecordStore',
    'RecordWriteStore',
    'Cursor',
    # agent
    'Agent',
    'Guarantee',
    'handle_event',
    'handle_request',
    'handle_response',
    # app
    'EventyApp',
    # errors
    'MessagingError',
]



# Copyright (c) Qotto, 2021

"""
Eventy aiohttp integration configuration
"""

__all__ = [
    'AIOHTTP_WEB_ACCESS_DISABLE_HEALTH_LOGGING',
    'AIOHTTP_WEB_ACCESS_HEALTH_ROUTE',
]

AIOHTTP_WEB_ACCESS_DISABLE_HEALTH_LOGGING: bool = True
"""
Disable aiohttp web server access logs for health checks
"""

AIOHTTP_WEB_ACCESS_HEALTH_ROUTE: str = '/health'
"""
Route for health checks
"""
